﻿using UnityEngine;
using System.Collections.Generic;
using Battlehub.RTCommon;

/* /////////////////////////////////////////////////////////////////////////////////////////////////

Author: PluginIdea 
Email:819661664@qq.com
Patreon:https://www.patreon.com/PluginIdea

Licensed under CC BY-SA after EarlyAccess ended. (see https://creativecommons.org/licenses/by-sa/4.0/)

///////////////////////////////////////////////////////////////////////////////////////////////// */

namespace PluginIdea
{
    public class HairHandle : MVRScript
    {

        private JSONStorableBool isWorldSpace;

        private List<FreeControllerV3> hairTools = new List<FreeControllerV3>();
        private int index = 0;
        private bool isShowAxis = true;

        public override void Init()
        {
            base.Init();

            UIInit();

            Transform rescaleObj = containingAtom.reParentObject.Find("object/rescaleObject");
            for (int i = 0; i < 4; i++)
            {
                Transform hairTool = rescaleObj.Find(string.Format("hairTool{0}", i + 1));
                if (hairTool != null)
                {
                    hairTools.Add(hairTool.GetComponent<FreeControllerV3>());
                }
            }
            FreeControllerV3 scapMask = rescaleObj.Find("hairScalpMaskTool").GetComponent<FreeControllerV3>();
            hairTools.Add(scapMask);

            SuperController.singleton.BroadcastMessage("OnActionsProviderAvailable", this, SendMessageOptions.DontRequireReceiver);

        }

        private void UIInit()
        {
            isWorldSpace = Utils.SetupToggle(this, "IsWorldSpace", false, false);

            Utils.SetupInfoText(this, "<size=40><b>HairHandle 1.0</b></size>\n" +
                "<b>J:</b> Next Hair Tool\n\n" +
                "<b>H:</b>Show/Hide Axis\n\n" +
                "You can use the Keybinding plugin to bind shortcut keys.", 300, true);
        }

        private bool IsHandlesEnable
        {
            get
            {
                SuperController sc = SuperController.singleton;
                UserPreferences up = UserPreferences.singleton;
                bool handlesEnabled = isShowAxis && sc.MonitorCenterCamera != null && (!up.showTargetsMenuOnly || sc.mainHUD.gameObject.activeSelf);
                return handlesEnabled;
            }
        }

        private void ShowRotateHandleView()
        {
            FCRotationHandle rotationHandle = SuperController.singleton.selectedControllerRotationHandle;
            rotationHandle.controller = hairTools[index];
            if (rotationHandle != null)
            {
                //if (rotationHandle.enabled && !IsHandlesEnable)
                //    rotationHandle.ForceDeselectAxis();
                rotationHandle.enabled = IsHandlesEnable;
            }
        }

        private void ShowPositionHandleView()
        {
            FCPositionHandle positionHandle = SuperController.singleton.selectedControllerPositionHandle;
            positionHandle.controller = hairTools[index];
            if (positionHandle != null)
            {
                //if (positionHandle.enabled && IsHandlesEnable)
                //    positionHandle.ForceDeselectAxis();
                positionHandle.enabled = IsHandlesEnable;
            }
        }

        private void Update()
        {
            RuntimeTools.PivotRotation = isWorldSpace.val ? RuntimePivotRotation.Global : RuntimePivotRotation.Local;

            if (Input.GetKeyDown(KeyCode.J))
            {
                SelectNextBrush();
            }

            if (Input.GetKeyDown(KeyCode.H))
            {
                ShowHideAxis();
            }

            ShowRotateHandleView();
            ShowPositionHandleView();
        }

        private void ShowHideAxis()
        {
            isShowAxis = !isShowAxis;
        }

        private void SelectNextBrush()
        {
            index++;
            if (index > hairTools.Count - 1)
                index = 0;
        }

        public void OnBindingsListRequested(List<object> bindings)
        {
            bindings.Add(new JSONStorableAction("Select NextBrush", SelectNextBrush));
            bindings.Add(new JSONStorableAction("ShowHide Axis", ShowHideAxis));
        }


        private void OnDestroy()
        {
            SuperController.singleton.BroadcastMessage("OnActionsProviderDestroyed", this, SendMessageOptions.DontRequireReceiver);
        }
    }
}
